/*
 * Tools for the creation of input file for CEMHYD3D v 3.0, XML file format
 * smilauer@cml.fsv.cvut.cz, 2010/05/06
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <rfftw.h>
#include "CEM_toolsXML_py.h"


CEM_toolsXML_py :: CEM_toolsXML_py(void) { }

CEM_toolsXML_py :: ~CEM_toolsXML_py(void) { }

//using namespace boost::python;

void CEM_toolsXML_py :: GenerateXMLInputCEMHYD(boost :: python :: list CEMHYD_variables, boost :: python :: list volume, boost :: python :: list CEMHYD_var_concrete, int Fineness_switch) {
    int i, length;
    int cnt = 0;
    double totsodium = 0., totpotassium = 0., rssodium = 0., rspotassium = 0.;


    //dynamic allocation and conversion for CEMHYD_variables
    length = boost :: python :: len(CEMHYD_variables);
    char ( * str ) [ 100 ] = new char [ length ] [ 100 ];
    for ( i = 0; i < length; i++ ) {
        strcpy( str [ i ], boost :: python :: extract< char * >(CEMHYD_variables [ i ]) );
        //printf("%s\n",str[i]);
    }

    //definitions of heat capacities [J/kg/K]
    double Cp_SCM = 700;
    double Cp_FA = 800;
    double Cp_CA = 800;
    double Cp_inert = 800;
    
    int rand_seed_num = atoi(str [ 0 ]);
    int SYSSIZE = atoi(str [ 1 ]);
    double dormant = atof(str [ 2 ]);
    double time_mapping = atof(str [ 3 ]);
    double fineness = atof(str [ 4 ]);
    double anhydrite = atof(str [ 5 ]);
    double hemihydrate = atof(str [ 6 ]);
    double dihydrate = atof(str [ 7 ]);
    double C3S_mass = atof(str [ 8 ]);
    double C2S_mass = atof(str [ 9 ]);
    double C3A_mass = atof(str [ 10 ]);
    double C4AF_mass = atof(str [ 11 ]);
    double wcr = atof(str [ 12 ]);
    double Ea_cem = atof(str [ 13 ]);
    double capacity_paste, capacity_concrete;
    //double temperature = atof(str[14]);
    char adiabatic_curing [ 16 ];
    strcpy(adiabatic_curing, str [ 15 ]);
    char AutoCorrFile [ 16 ];
    strcpy(AutoCorrFile, str [ 16 ]);
    char Sat_seal [ 16 ];
    strcpy(Sat_seal, str [ 17 ]);
    delete [] str;

    //dynamic allocation and conversion for CEMHYD_var_concrete
    length = boost :: python :: len(CEMHYD_var_concrete);
    char ( * str1 ) [ 100 ] = new char [ length ] [ 100 ];
    for ( i = 0; i < length; i++ ) {
        strcpy( str1 [ i ], boost :: python :: extract< char * >(CEMHYD_var_concrete [ i ]) );
        //printf("%s\n",str1[i]);
    }

    double mass_cem = atof(str1 [ 0 ]);
    double mass_SCM = atof(str1 [ 1 ]);
    double mass_FA = atof(str1 [ 2 ]);
    double mass_CA = atof(str1 [ 3 ]);
    double mass_inert = atof(str1 [ 4 ]);
    double E_SCM = atof(str1 [ 10 ]);
    double E_FA = atof(str1 [ 11 ]);
    double E_CA = atof(str1 [ 12 ]);
    double E_inert = atof(str1 [ 13 ]);
    double nu_SCM = atof(str1 [ 14 ]);
    double nu_FA = atof(str1 [ 15 ]);
    double nu_CA = atof(str1 [ 16 ]);
    double nu_inert = atof(str1 [ 17 ]);
    double grain_FA = atof(str1 [ 18 ]);
    double grain_CA = atof(str1 [ 19 ]);
    double vol_entrained_entrapped = atof(str1 [ 20 ]);
    double ITZ_thick = atof(str1 [ 21 ]);
    double ITZ_Young_red = atof(str1 [ 22 ]);
    double Cp_cem = atof(str1 [ 23 ]);
    double Cp_agg  = atof(str1 [ 24 ]);
    delete [] str1;

    //conversion for volume
    double vol_cem = boost :: python :: extract< double const >(volume [ 0 ]);
    double vol_SCM = boost :: python :: extract< double const >(volume [ 1 ]);
    double vol_wat = mass_cem * wcr;
    double vol_FA = boost :: python :: extract< double const >(volume [ 2 ]);
    double vol_CA = boost :: python :: extract< double const >(volume [ 3 ]);
    double vol_inert = boost :: python :: extract< double const >(volume [ 4 ]);

    //printf("w %f\n", vol_wat);

    //used in input file generation
    double mass_tot = mass_cem + mass_SCM + vol_wat + mass_FA + mass_CA + mass_inert;
    double vol_tot = vol_cem + vol_SCM + vol_wat + vol_FA + vol_CA + vol_inert + vol_entrained_entrapped;
    
    double volume_clinker;
    double clinker_unity_frac [ 4 ];
    double spec_mass [ 7 ];
    double vol_with_gypsum [ 7 ];
    double cem_density_with_gypsum = 0.;
    double mass_frac_wcr, vol_frac_wcr;
    double n, b;
    double PSD [ 40 ];
    int max_radius, truncated_radius, satur_sealed;
    double counter, truncated_rel_vol, vol_placed;
    long int Particles [ 40 ];

    long int volpart [ 40 ];
    long int total_voxels;
    struct Autocorr_data
    {
        double sil [ 60 ];
        double c3s [ 60 ];
        double c3a [ 60 ];
    };
    struct Autocorr_data *p_autocorr = NULL;

    //convert from percent
    anhydrite /= 100.;
    hemihydrate /= 100.;
    dihydrate /= 100.;
    C3S_mass /= 100.;
    C2S_mass /= 100.;
    C3A_mass /= 100.;
    C4AF_mass /= 100.;


    //voxels per particle radius
    volpart [ 0 ] = 1;
    volpart [ 1 ] = 19;
    volpart [ 2 ] = 81;
    volpart [ 3 ] = 179;
    volpart [ 4 ] = 389;
    volpart [ 5 ] = 739;
    volpart [ 6 ] = 1189;
    volpart [ 7 ] = 1791;
    volpart [ 8 ] = 2553;
    volpart [ 9 ] = 3695;
    volpart [ 10 ] = 4945;
    volpart [ 11 ] = 6403;
    volpart [ 12 ] = 8217;
    volpart [ 13 ] = 10395;
    volpart [ 14 ] = 12893;
    volpart [ 15 ] = 15515;
    volpart [ 16 ] = 18853;
    volpart [ 17 ] = 22575;
    volpart [ 18 ] = 26745;
    volpart [ 19 ] = 31103;
    volpart [ 20 ] = 36137;
    volpart [ 21 ] = 41851;
    volpart [ 22 ] = 47833;
    volpart [ 23 ] = 54435;
    volpart [ 24 ] = 61565;
    volpart [ 25 ] = 69599;
    volpart [ 26 ] = 78205;
    volpart [ 27 ] = 87271;
    volpart [ 28 ] = 97233;
    volpart [ 29 ] = 107783;
    volpart [ 30 ] = 119009;
    volpart [ 31 ] = 131155;
    volpart [ 32 ] = 143761;
    volpart [ 33 ] = 157563;
    volpart [ 34 ] = 172317;
    volpart [ 35 ] = 187511;
    volpart [ 36 ] = 203965;

    //specific mass of C3S, C2S, C3A, C4AF, dihydrate, hemihydrate, anhydrite
    spec_mass [ 0 ] = 3.21;
    spec_mass [ 1 ] = 3.28;
    spec_mass [ 2 ] = 3.03;
    spec_mass [ 3 ] = 3.73;
    spec_mass [ 4 ] = 2.32;
    spec_mass [ 5 ] = 2.73;
    spec_mass [ 6 ] = 2.61;


    for ( int i = 0; i < 40; i++ ) {
        Particles [ i ] = 0;
    }


    if ( strcmp(Sat_seal, "Saturated") == 0 ) {
        satur_sealed = 0;
    } else {
        satur_sealed = 1;
    }

    //calculate volume of all 4 clinker minerals
    volume_clinker = C3S_mass / spec_mass [ 0 ] + C2S_mass / spec_mass [ 1 ] + C3A_mass / spec_mass [ 2 ] + C4AF_mass / spec_mass [ 3 ];

    clinker_unity_frac [ 0 ] = C3S_mass / spec_mass [ 0 ] / volume_clinker;
    clinker_unity_frac [ 1 ] = C2S_mass / spec_mass [ 1 ] / volume_clinker;
    clinker_unity_frac [ 2 ] = C3A_mass / spec_mass [ 2 ] / volume_clinker;
    clinker_unity_frac [ 3 ] = C4AF_mass / spec_mass [ 3 ] / volume_clinker;

    if ( clinker_unity_frac [ 0 ] +  clinker_unity_frac [ 1 ] + clinker_unity_frac [ 2 ] + clinker_unity_frac [ 3 ] < 0.99 || clinker_unity_frac [ 0 ] +  clinker_unity_frac [ 1 ] + clinker_unity_frac [ 2 ] + clinker_unity_frac [ 3 ] > 1.01 ) {
        printf("Clinker volumetric fractions are not 1.0 (file %s, line %d)\n", __FILE__, __LINE__);
        exit(1);
    }

    for ( int i = 0; i < 4; i++ ) {
        vol_with_gypsum [ i ] = clinker_unity_frac [ i ] * ( 1. - dihydrate -  hemihydrate - anhydrite );
    }

    vol_with_gypsum [ 4 ] = dihydrate;
    vol_with_gypsum [ 5 ] = hemihydrate;
    vol_with_gypsum [ 6 ] = anhydrite;

    for ( int i = 0; i < 7; i++ ) {
        cem_density_with_gypsum += spec_mass [ i ] *  vol_with_gypsum [ i ];
    }

    //calculate mass and volume fracion of water in the RVE
    mass_frac_wcr = wcr / ( 1. + wcr );
    vol_frac_wcr = mass_frac_wcr / ( mass_frac_wcr + ( 1. - mass_frac_wcr ) / cem_density_with_gypsum );

    total_voxels = ( long int ) ( ( 1. - vol_frac_wcr ) * SYSSIZE * SYSSIZE * SYSSIZE );
    //printf("Total voxels to be placed %ld\n",  total_voxels);

    //find volume (mass) fractions of individual radii
    //German cements - PSD
    if ( Fineness_switch == 0 ) { //Dyckerhoff fitted from NIST database
        n = -0.000833 * fineness + 1.1175;
        b = 0.000754 * fineness - 0.143;
    } else {
        //according to van Breugel
        n = -0.00031 * fineness + 1.2;
        b = 0.000174 * fineness - 0.029;
    }

    //printf("n %f b %f\n", n,b);

    for ( int i = 0; i < 40; i++ ) {
        PSD [ i ] = 1. - exp( -b * pow( ( 2. * i + 1. ), n ) );
    }

    //determine maximum radius that can be teoretically used for given SYSSIZE
    max_radius = int ( SYSSIZE / 3. );

    /*
     *
     * ------0-------1-------2-----3---..---------truncated_radius--------> radius
     *
     */

    //find the maximum size of spare particle that substitutes the rest of larger particles that can not be placed inside the RVE
    counter = 0.;
    truncated_rel_vol = 0.;

    for ( int i = 39; i >= 1; i-- ) {
        if ( i <= max_radius && ( 1 - PSD [ i - 1 ] ) * total_voxels / volpart [ i ] > 0.5 ) {
            //printf("PSD truncated at radius %d\n", i);
            truncated_radius = i;
            Particles [ i ] = ( long int ) round( ( 1 - PSD [ i - 1 ] ) * total_voxels / volpart [ i ] );
            if ( Particles [ i ] == 0 ) {
                Particles [ i ] = 1;
            }

            vol_placed = Particles [ i ] * volpart [ i ];
            //printf("r %d n %d vol_placed %f remaining %f\n", i, Particles[i], vol_placed, (total_voxels-vol_placed)/total_voxels);
            break;
        }
    }

    //place particles bellow truncated radius
    for ( int i = truncated_radius - 1; i >= 0; i-- ) {
        counter = ( ( 1 - ( i == 0 ? 0. : PSD [ i - 1 ] ) ) * total_voxels - vol_placed ) / volpart [ i ];
        if ( counter >= 0.5 ) {
            Particles [ i ] = ( long int ) round(counter);
            vol_placed += Particles [ i ] * volpart [ i ];
            //printf("r %d n %d vol_placed %f remaining %f\n", i, Particles[i], vol_placed, (total_voxels-vol_placed)/total_voxels);
        }
    }

    //Autocorrelation files
    struct Autocorr_data D16130s1 =
    { {
          0.399102, 0.286178, 0.241162, 0.215826, 0.201068, 0.191144, 0.184427, 0.179909, 0.176856, 0.174323, 0.172522, 0.171031, 0.169673, 0.168590, 0.166987, 0.165772, 0.165030, 0.164139, 0.163064, 0.162240, 0.161285, 0.160338, 0.159571, 0.158950, 0.159032, 0.159186, 0.159155, 0.158841, 0.158348, 0.157817, 0.157547, 0.157410, 0.157558, 0.157642, 0.157332, 0.156982, 0.156963, 0.157247, 0.157462, 0.157313, 0.156780, 0.155999, 0.155570, 0.154809, 0.154124, 0.153528, 0.153073, 0.152811, 0.153074, 0.153318, 0.153606, 0.153813, 0.153935, 0.153765, 0.153514, 0.153264, 0.153224, 0.153356, 0.153439, 0.153537
      },
      { 0.380117, 0.274355, 0.229658, 0.204081, 0.189174, 0.179122, 0.172566, 0.168099, 0.164928, 0.162307, 0.160478, 0.158898, 0.157436, 0.156120, 0.154501, 0.153144, 0.152169, 0.151126, 0.149901, 0.148864, 0.147663, 0.146373, 0.145395, 0.144761, 0.144781, 0.144801, 0.144665, 0.144226, 0.143542, 0.142854, 0.142424, 0.142214, 0.142303, 0.142419, 0.142087, 0.141768, 0.141686, 0.141769, 0.141837, 0.141598, 0.141063, 0.140296, 0.139752, 0.138966, 0.138285, 0.137617, 0.137071, 0.136737, 0.136910, 0.137063, 0.137422, 0.137692, 0.137705, 0.137481, 0.137103, 0.136726, 0.136600, 0.136629, 0.136559, 0.136526 },
      {
          0.041699, 0.015764, 0.010158, 0.007978, 0.006743, 0.006063, 0.005742, 0.005416, 0.005162, 0.004962, 0.004793, 0.004593, 0.004292, 0.004092, 0.003850, 0.003606, 0.003429, 0.003306, 0.003131, 0.003045, 0.002885, 0.002768, 0.002671, 0.002518, 0.002431, 0.002352, 0.002237, 0.002178, 0.002121, 0.002057, 0.001964, 0.001923, 0.001863, 0.001825, 0.001807, 0.001792, 0.001798, 0.001829, 0.001856, 0.001869, 0.001877, 0.001852, 0.001787, 0.001758, 0.001747, 0.001754, 0.001768, 0.001776, 0.001814, 0.001842, 0.001852, 0.001919, 0.001968, 0.001985, 0.001983, 0.002017, 0.002023, 0.002013, 0.002044, 0.002055
      } };

    struct Autocorr_data D16131s1 =
    { {
          0.379648, 0.281362, 0.236394, 0.209773, 0.194063, 0.183225, 0.175171, 0.169513, 0.165863, 0.162820, 0.159867, 0.157083, 0.155022, 0.153456, 0.152133, 0.150932, 0.149704, 0.148642, 0.148068, 0.147888, 0.147548, 0.147229, 0.147013, 0.147228, 0.147505, 0.147453, 0.147403, 0.147126, 0.146842, 0.146738, 0.146870, 0.147110, 0.147171, 0.147105, 0.147077, 0.147183, 0.147369, 0.147348, 0.147388, 0.147529, 0.147418, 0.147266, 0.147257, 0.147347, 0.147659, 0.148069, 0.148323, 0.148332, 0.148329, 0.148191, 0.148109, 0.148352, 0.148533, 0.148639, 0.148975, 0.149476, 0.149969, 0.150330, 0.150572, 0.150917
      },
      { 0.357617, 0.266091, 0.221810, 0.194172, 0.177496, 0.166305, 0.158323, 0.152626, 0.148709, 0.145379, 0.142410, 0.139572, 0.137501, 0.136212, 0.135193, 0.134241, 0.133114, 0.132195, 0.131719, 0.131522, 0.131144, 0.131029, 0.130943, 0.131145, 0.131502, 0.131634, 0.131717, 0.131577, 0.131346, 0.131255, 0.131125, 0.131090, 0.131119, 0.131046, 0.130968, 0.130893, 0.130840, 0.130678, 0.130640, 0.130759, 0.130565, 0.130392, 0.130427, 0.130522, 0.130796, 0.131119, 0.131320, 0.131276, 0.131207, 0.131014, 0.130953, 0.131177, 0.131346, 0.131497, 0.131877, 0.132401, 0.132967, 0.133351, 0.133585, 0.133882 },
      {
          0.057480, 0.025807, 0.015534, 0.010754, 0.007961, 0.006312, 0.005157, 0.004481, 0.004065, 0.003760, 0.003701, 0.003776, 0.003790, 0.003657, 0.003555, 0.003477, 0.003496, 0.003542, 0.003556, 0.003507, 0.003509, 0.003543, 0.003665, 0.003790, 0.003832, 0.003761, 0.003657, 0.003547, 0.003503, 0.003446, 0.003402, 0.003374, 0.003435, 0.003467, 0.003432, 0.003446, 0.003400, 0.003347, 0.003307, 0.003299, 0.003376, 0.003450, 0.003497, 0.003515, 0.003557, 0.003558, 0.003619, 0.003664, 0.003680, 0.003605, 0.003561, 0.003542, 0.003509, 0.003424, 0.003330, 0.003252, 0.003226, 0.003298, 0.003340, 0.003332
      } };

    struct Autocorr_data D16132s1 =
    { {
          0.380664, 0.301057, 0.258545, 0.229457, 0.209290, 0.195534, 0.185625, 0.177155, 0.170464, 0.165091, 0.161094, 0.158313, 0.156051, 0.154078, 0.152068, 0.150201, 0.148814, 0.148049, 0.147361, 0.146376, 0.145374, 0.144467, 0.143742, 0.143209, 0.142634, 0.142029, 0.141303, 0.140645, 0.140230, 0.140130, 0.140128, 0.140357, 0.140595, 0.140705, 0.140902, 0.141216, 0.141699, 0.142149, 0.142537, 0.143158, 0.144030, 0.144879, 0.145285, 0.145499, 0.145713, 0.145578, 0.145487, 0.145659, 0.146232, 0.147019, 0.147982, 0.148760, 0.149082, 0.149290, 0.149307, 0.149286, 0.149426, 0.149564, 0.149692, 0.149680
      },
      { 0.364688, 0.288014, 0.246615, 0.217907, 0.197913, 0.184015, 0.173838, 0.165429, 0.158741, 0.153401, 0.149308, 0.146425, 0.144161, 0.142289, 0.140419, 0.138729, 0.137411, 0.136577, 0.135865, 0.134983, 0.133962, 0.132914, 0.132053, 0.131419, 0.130815, 0.130175, 0.129534, 0.128972, 0.128526, 0.128392, 0.128428, 0.128660, 0.128862, 0.128922, 0.129073, 0.129379, 0.129871, 0.130339, 0.130728, 0.131360, 0.132198, 0.132998, 0.133395, 0.133562, 0.133662, 0.133413, 0.133230, 0.133358, 0.133917, 0.134765, 0.135759, 0.136551, 0.136932, 0.137137, 0.137141, 0.137252, 0.137474, 0.137642, 0.137708, 0.137644 },
      {
          0.043945, 0.020624, 0.013169, 0.009402, 0.007393, 0.006173, 0.005148, 0.004475, 0.004084, 0.003629, 0.003139, 0.002821, 0.002598, 0.002474, 0.002404, 0.002368, 0.002314, 0.002202, 0.002110, 0.002012, 0.001946, 0.001937, 0.001896, 0.001921, 0.001912, 0.001836, 0.001833, 0.001895, 0.001968, 0.002012, 0.002049, 0.002034, 0.001982, 0.001933, 0.001875, 0.001827, 0.001818, 0.001845, 0.001833, 0.001859, 0.001859, 0.001809, 0.001761, 0.001734, 0.001759, 0.001772, 0.001808, 0.001824, 0.001792, 0.001770, 0.001749, 0.001763, 0.001749, 0.001787, 0.001826, 0.001866, 0.001893, 0.001931, 0.001961, 0.001955
      } };

    struct Autocorr_data D16133s1 =
    { {
          0.431309, 0.345241, 0.301297, 0.273299, 0.255656, 0.243758, 0.234843, 0.228292, 0.223525, 0.219477, 0.216010, 0.213360, 0.211216, 0.209817, 0.208382, 0.206988, 0.205992, 0.205192, 0.204909, 0.204750, 0.204250, 0.203546, 0.202777, 0.201833, 0.200837, 0.199912, 0.199160, 0.198421, 0.197739, 0.196990, 0.196209, 0.195748, 0.195126, 0.194593, 0.194194, 0.193577, 0.192863, 0.192291, 0.191657, 0.191220, 0.190953, 0.190611, 0.190196, 0.189702, 0.189000, 0.188440, 0.187908, 0.187440, 0.187038, 0.186744, 0.186821, 0.187107, 0.187351, 0.187659, 0.188241, 0.188969, 0.189504, 0.189654, 0.189563, 0.189432
      },
      { 0.402148, 0.318547, 0.275058, 0.247104, 0.229583, 0.217428, 0.208580, 0.202295, 0.197610, 0.193542, 0.190159, 0.187481, 0.185353, 0.183878, 0.182546, 0.181273, 0.180200, 0.179495, 0.179309, 0.179302, 0.179162, 0.178973, 0.178492, 0.177716, 0.176911, 0.176176, 0.175499, 0.174910, 0.174477, 0.173887, 0.173201, 0.172687, 0.172069, 0.171531, 0.171138, 0.170465, 0.169687, 0.169114, 0.168558, 0.168060, 0.167626, 0.167167, 0.166739, 0.166156, 0.165371, 0.164684, 0.164248, 0.163800, 0.163401, 0.163042, 0.162923, 0.163043, 0.163283, 0.163589, 0.164136, 0.164732, 0.165159, 0.165258, 0.165220, 0.165066 },
      {
          0.055215, 0.023773, 0.013803, 0.009514, 0.007362, 0.005947, 0.005025, 0.004382, 0.004016, 0.003792, 0.003536, 0.003349, 0.003308, 0.003433, 0.003533, 0.003485, 0.003425, 0.003399, 0.003203, 0.003125, 0.003151, 0.003110, 0.003126, 0.003134, 0.003039, 0.003030, 0.003034, 0.003003, 0.002933, 0.002878, 0.002827, 0.002860, 0.002822, 0.002815, 0.002860, 0.002918, 0.002996, 0.002959, 0.002908, 0.002903, 0.002912, 0.002909, 0.002905, 0.002981, 0.003014, 0.002950, 0.002855, 0.002767, 0.002775, 0.002829, 0.002865, 0.002875, 0.002881, 0.002889, 0.002883, 0.002846, 0.002812, 0.002772, 0.002753, 0.002695
      } };

    struct Autocorr_data D16134s1 =
    { {
          0.416562, 0.340707, 0.298764, 0.269866, 0.249325, 0.234548, 0.223984, 0.216077, 0.209965, 0.205302, 0.201635, 0.198549, 0.195352, 0.192329, 0.189343, 0.186718, 0.184546, 0.182574, 0.180515, 0.179090, 0.178124, 0.177408, 0.176856, 0.176360, 0.176247, 0.176515, 0.176659, 0.176358, 0.175544, 0.174708, 0.173799, 0.173392, 0.173159, 0.173214, 0.173815, 0.174549, 0.175299, 0.176077, 0.176927, 0.177934, 0.178859, 0.179681, 0.180430, 0.181111, 0.181683, 0.182287, 0.182830, 0.183021, 0.182853, 0.182545, 0.182505, 0.182862, 0.183435, 0.183972, 0.184322, 0.184535, 0.184547, 0.184345, 0.183947, 0.183316
      },
      { 0.408770, 0.334832, 0.293212, 0.264377, 0.243889, 0.229167, 0.218660, 0.210772, 0.204635, 0.199725, 0.195824, 0.192536, 0.189230, 0.186104, 0.183043, 0.180369, 0.178084, 0.176071, 0.174026, 0.172544, 0.171476, 0.170595, 0.169939, 0.169391, 0.169292, 0.169535, 0.169682, 0.169396, 0.168557, 0.167726, 0.166842, 0.166503, 0.166276, 0.166297, 0.166869, 0.167607, 0.168319, 0.169065, 0.169923, 0.170912, 0.171786, 0.172509, 0.173193, 0.173827, 0.174415, 0.174975, 0.175411, 0.175581, 0.175351, 0.174938, 0.174866, 0.175192, 0.175701, 0.176200, 0.176595, 0.176808, 0.176804, 0.176629, 0.176205, 0.175514 },
      {
          0.043242, 0.018859, 0.011718, 0.008223, 0.006214, 0.004917, 0.003937, 0.003288, 0.002936, 0.002630, 0.002305, 0.002043, 0.001847, 0.001861, 0.001884, 0.001861, 0.001869, 0.001844, 0.001836, 0.001845, 0.001954, 0.002098, 0.002257, 0.002352, 0.002359, 0.002343, 0.002270, 0.002257, 0.002182, 0.002065, 0.001965, 0.001916, 0.001934, 0.001960, 0.001960, 0.001954, 0.001970, 0.001940, 0.001906, 0.001891, 0.001955, 0.001996, 0.002035, 0.002016, 0.001976, 0.001979, 0.002029, 0.002063, 0.002052, 0.002043, 0.001977, 0.001949, 0.001937, 0.001944, 0.001935, 0.001906, 0.001935, 0.001933, 0.001918, 0.001877
      } };

    struct Autocorr_data D16148s1 =
    { {
          0.405801, 0.310089, 0.267253, 0.241361, 0.224010, 0.211772, 0.203292, 0.197527, 0.193219, 0.189904, 0.186603, 0.183665, 0.181230, 0.179198, 0.177787, 0.176854, 0.175863, 0.174976, 0.174332, 0.174220, 0.174067, 0.174123, 0.174546, 0.174768, 0.174697, 0.174721, 0.174866, 0.174584, 0.173854, 0.173280, 0.172727, 0.172062, 0.171407, 0.170870, 0.170715, 0.170730, 0.170580, 0.170291, 0.170164, 0.170098, 0.170006, 0.170137, 0.170276, 0.170517, 0.170816, 0.170905, 0.170901, 0.170948, 0.171275, 0.171604, 0.171930, 0.172386, 0.172740, 0.173197, 0.173821, 0.174459, 0.174950, 0.175521, 0.176167, 0.176830
      },
      { 0.338652, 0.249585, 0.210934, 0.187749, 0.172040, 0.160680, 0.152712, 0.147272, 0.142987, 0.139593, 0.136425, 0.133560, 0.131289, 0.129529, 0.128096, 0.127110, 0.126229, 0.125421, 0.124826, 0.124612, 0.124287, 0.124066, 0.123966, 0.123702, 0.123076, 0.122664, 0.122393, 0.121942, 0.121432, 0.121055, 0.120697, 0.120315, 0.119812, 0.119270, 0.118893, 0.118774, 0.118650, 0.118499, 0.118471, 0.118548, 0.118567, 0.118662, 0.118707, 0.118885, 0.119108, 0.119073, 0.118918, 0.118750, 0.118853, 0.119131, 0.119422, 0.119781, 0.120089, 0.120420, 0.120840, 0.121284, 0.121642, 0.122084, 0.122691, 0.123270 },
      {
          0.026035, 0.009676, 0.005072, 0.002975, 0.002001, 0.001483, 0.001166, 0.000942, 0.000846, 0.000790, 0.000703, 0.000680, 0.000661, 0.000634, 0.000642, 0.000584, 0.000569, 0.000569, 0.000572, 0.000584, 0.000631, 0.000712, 0.000767, 0.000750, 0.000718, 0.000682, 0.000653, 0.000663, 0.000669, 0.000655, 0.000622, 0.000594, 0.000586, 0.000615, 0.000631, 0.000647, 0.000640, 0.000625, 0.000626, 0.000602, 0.000635, 0.000641, 0.000660, 0.000676, 0.000672, 0.000659, 0.000648, 0.000655, 0.000673, 0.000688, 0.000695, 0.000696, 0.000717, 0.000709, 0.000687, 0.000679, 0.000682, 0.000647, 0.000638, 0.000653
      } };

    struct Autocorr_data D16149s1 =
    { {
          0.400586, 0.307025, 0.263729, 0.236249, 0.217939, 0.205055, 0.194930, 0.187705, 0.182404, 0.178185, 0.175158, 0.173124, 0.171533, 0.170364, 0.169407, 0.168753, 0.168230, 0.167954, 0.168036, 0.168568, 0.168839, 0.168628, 0.168158, 0.168062, 0.167748, 0.167524, 0.167550, 0.167887, 0.168502, 0.168755, 0.169057, 0.169324, 0.169858, 0.170333, 0.170748, 0.171126, 0.171448, 0.171620, 0.171840, 0.171815, 0.171753, 0.171746, 0.171511, 0.171030, 0.170544, 0.169998, 0.169729, 0.169409, 0.169401, 0.169602, 0.169809, 0.169963, 0.169823, 0.169670, 0.169518, 0.169584, 0.169743, 0.169744, 0.169647, 0.169632
      },
      { 0.345996, 0.255922, 0.216158, 0.191612, 0.174888, 0.163178, 0.154300, 0.147894, 0.143316, 0.139686, 0.137092, 0.135365, 0.133829, 0.132500, 0.131307, 0.130341, 0.129265, 0.128392, 0.127954, 0.127908, 0.127722, 0.127157, 0.126311, 0.125690, 0.125116, 0.124848, 0.124848, 0.124897, 0.125167, 0.125400, 0.125607, 0.125770, 0.126127, 0.126427, 0.126808, 0.127253, 0.127732, 0.127986, 0.128373, 0.128796, 0.129109, 0.129367, 0.129340, 0.129078, 0.128768, 0.128363, 0.128168, 0.127835, 0.127676, 0.127712, 0.127810, 0.127905, 0.127771, 0.127757, 0.127882, 0.128071, 0.128181, 0.128202, 0.128136, 0.128098 },
      {
          0.033867, 0.011443, 0.006540, 0.004458, 0.003274, 0.002456, 0.001949, 0.001638, 0.001563, 0.001470, 0.001427, 0.001381, 0.001325, 0.001345, 0.001373, 0.001362, 0.001263, 0.001169, 0.001168, 0.001177, 0.001166, 0.001119, 0.001095, 0.001091, 0.001077, 0.001103, 0.001134, 0.001178, 0.001161, 0.001128, 0.001101, 0.001051, 0.001046, 0.001032, 0.001034, 0.001034, 0.001037, 0.001064, 0.001080, 0.001070, 0.001065, 0.001061, 0.001057, 0.001088, 0.001078, 0.001074, 0.001087, 0.001091, 0.001097, 0.001102, 0.001081, 0.001090, 0.001078, 0.001047, 0.001022, 0.001036, 0.001047, 0.001071, 0.001093, 0.001087
      } };

    struct Autocorr_data D16150s1 =
    { {
          0.375977, 0.301951, 0.262861, 0.235539, 0.216718, 0.203123, 0.193091, 0.184855, 0.178187, 0.172536, 0.167797, 0.163851, 0.160210, 0.156859, 0.153850, 0.151534, 0.149878, 0.148588, 0.147660, 0.147104, 0.146644, 0.146326, 0.145914, 0.145743, 0.145659, 0.145487, 0.145099, 0.144438, 0.143712, 0.143013, 0.142265, 0.141427, 0.140488, 0.139588, 0.138690, 0.137811, 0.137127, 0.136626, 0.136360, 0.136449, 0.136967, 0.137704, 0.138444, 0.139187, 0.139871, 0.140666, 0.141189, 0.141659, 0.142182, 0.142955, 0.143715, 0.144514, 0.145155, 0.145573, 0.145791, 0.145832, 0.145818, 0.145615, 0.145544, 0.145569
      },
      { 0.283828, 0.213692, 0.179658, 0.157404, 0.141837, 0.130355, 0.121813, 0.115148, 0.109561, 0.104833, 0.100926, 0.097587, 0.094527, 0.091735, 0.089217, 0.087456, 0.086116, 0.084906, 0.084084, 0.083604, 0.083168, 0.082819, 0.082432, 0.082317, 0.082399, 0.082414, 0.082278, 0.081837, 0.081069, 0.080374, 0.079715, 0.078956, 0.077956, 0.076890, 0.075756, 0.074796, 0.074123, 0.073557, 0.073030, 0.072730, 0.072680, 0.072773, 0.072870, 0.072964, 0.072952, 0.072933, 0.072898, 0.073086, 0.073387, 0.073938, 0.074718, 0.075468, 0.076214, 0.076780, 0.077155, 0.077510, 0.077680, 0.077610, 0.077559, 0.077601 },
      {
          0.033730, 0.011081, 0.006497, 0.004488, 0.003506, 0.002810, 0.002389, 0.002111, 0.001870, 0.001659, 0.001472, 0.001375, 0.001308, 0.001213, 0.001187, 0.001238, 0.001214, 0.001234, 0.001204, 0.001181, 0.001182, 0.001111, 0.001067, 0.001046, 0.001077, 0.001091, 0.001073, 0.001028, 0.001027, 0.001016, 0.001022, 0.001012, 0.000951, 0.000921, 0.000924, 0.000912, 0.000899, 0.000903, 0.000915, 0.000931, 0.000937, 0.000983, 0.001013, 0.000990, 0.000978, 0.000996, 0.000993, 0.001018, 0.001027, 0.001033, 0.001073, 0.001097, 0.001108, 0.001102, 0.001087, 0.001114, 0.001139, 0.001157, 0.001198, 0.001202
      } };

    struct Autocorr_data D16155s1 =
    { {
          0.439297, 0.370478, 0.329956, 0.300530, 0.279240, 0.263304, 0.251356, 0.242164, 0.234968, 0.229024, 0.224362, 0.220519, 0.217520, 0.214762, 0.212485, 0.210716, 0.209687, 0.209145, 0.208691, 0.208542, 0.208787, 0.209044, 0.209400, 0.209877, 0.210633, 0.211661, 0.212536, 0.212931, 0.213076, 0.213083, 0.213039, 0.212995, 0.213049, 0.212985, 0.212564, 0.211991, 0.211430, 0.211141, 0.210681, 0.209907, 0.209043, 0.208225, 0.207386, 0.206584, 0.205816, 0.205291, 0.205050, 0.204825, 0.204567, 0.204181, 0.203896, 0.203563, 0.203369, 0.203159, 0.203217, 0.203466, 0.203744, 0.204004, 0.204188, 0.204326
      },
      { 0.387207, 0.316960, 0.277791, 0.250571, 0.231256, 0.217060, 0.206749, 0.198847, 0.192444, 0.186886, 0.182518, 0.179008, 0.176112, 0.173406, 0.171303, 0.169631, 0.168526, 0.167883, 0.167364, 0.167246, 0.167457, 0.167531, 0.167760, 0.168101, 0.168704, 0.169520, 0.170135, 0.170064, 0.169699, 0.169367, 0.169201, 0.169229, 0.169459, 0.169528, 0.169215, 0.168801, 0.168414, 0.168194, 0.167842, 0.167140, 0.166249, 0.165433, 0.164629, 0.163757, 0.162844, 0.162092, 0.161621, 0.161267, 0.160871, 0.160413, 0.160071, 0.159624, 0.159193, 0.158728, 0.158431, 0.158318, 0.158257, 0.158216, 0.158172, 0.158211 },
      {
          0.053984, 0.026436, 0.017685, 0.013157, 0.010536, 0.008655, 0.007463, 0.006432, 0.005650, 0.005154, 0.004822, 0.004555, 0.004446, 0.004465, 0.004489, 0.004480, 0.004458, 0.004404, 0.004412, 0.004355, 0.004240, 0.004207, 0.004230, 0.004246, 0.004198, 0.004119, 0.004071, 0.004012, 0.003922, 0.003871, 0.003821, 0.003747, 0.003602, 0.003498, 0.003491, 0.003488, 0.003511, 0.003484, 0.003390, 0.003297, 0.003237, 0.003188, 0.003141, 0.003189, 0.003252, 0.003314, 0.003340, 0.003297, 0.003284, 0.003325, 0.003331, 0.003333, 0.003298, 0.003206, 0.003207, 0.003205, 0.003243, 0.003219, 0.003217, 0.003193
      } };

    struct Autocorr_data D16594s1 =
    { {
          0.335879, 0.223895, 0.183838, 0.163068, 0.150904, 0.143441, 0.138366, 0.134114, 0.131186, 0.129097, 0.127214, 0.125614, 0.124208, 0.123133, 0.122469, 0.121779, 0.121121, 0.120226, 0.119194, 0.118440, 0.117978, 0.117963, 0.118032, 0.117752, 0.117781, 0.117399, 0.117104, 0.116838, 0.116817, 0.116532, 0.116590, 0.116621, 0.116714, 0.116868, 0.116902, 0.116900, 0.116970, 0.116913, 0.116991, 0.116953, 0.116962, 0.117101, 0.117153, 0.117170, 0.116938, 0.116833, 0.116989, 0.117211, 0.116965, 0.116455, 0.116076, 0.115700, 0.115338, 0.114927, 0.114367, 0.113989, 0.113856, 0.113620, 0.113181, 0.112731
      },
      { 0.283379, 0.185513, 0.148160, 0.128334, 0.116742, 0.109189, 0.104534, 0.100906, 0.098374, 0.096548, 0.095067, 0.094022, 0.092979, 0.092064, 0.091583, 0.091002, 0.090337, 0.089391, 0.088247, 0.087283, 0.086536, 0.086418, 0.086424, 0.086228, 0.086299, 0.086075, 0.085648, 0.085144, 0.084947, 0.084625, 0.084444, 0.084339, 0.084409, 0.084343, 0.084197, 0.083866, 0.083630, 0.083565, 0.083667, 0.083606, 0.083627, 0.083763, 0.083819, 0.083719, 0.083647, 0.083791, 0.084101, 0.084323, 0.084331, 0.084127, 0.084006, 0.083808, 0.083622, 0.083298, 0.082936, 0.082694, 0.082546, 0.082375, 0.081986, 0.081582 },
      {
          0.017246, 0.004847, 0.002207, 0.001234, 0.000797, 0.000682, 0.000586, 0.000557, 0.000557, 0.000492, 0.000468, 0.000459, 0.000403, 0.000379, 0.000337, 0.000336, 0.000313, 0.000304, 0.000325, 0.000351, 0.000364, 0.000366, 0.000324, 0.000323, 0.000357, 0.000341, 0.000323, 0.000316, 0.000322, 0.000326, 0.000325, 0.000331, 0.000321, 0.000312, 0.000294, 0.000313, 0.000328, 0.000348, 0.000361, 0.000361, 0.000330, 0.000347, 0.000361, 0.000363, 0.000376, 0.000357, 0.000351, 0.000337, 0.000365, 0.000378, 0.000359, 0.000346, 0.000336, 0.000338, 0.000325, 0.000333, 0.000345, 0.000360, 0.000371, 0.000353
      } };

    if ( !strcmp(AutoCorrFile, "16130s1") ) {
        p_autocorr = & D16130s1;
    } else if ( !strcmp(AutoCorrFile, "16131s1") ) {
        p_autocorr = & D16131s1;
    } else if ( !strcmp(AutoCorrFile, "16132s1") ) {
        p_autocorr = & D16132s1;
    } else if ( !strcmp(AutoCorrFile, "16133s1") ) {
        p_autocorr = & D16133s1;
    } else if ( !strcmp(AutoCorrFile, "16134s1") ) {
        p_autocorr = & D16134s1;
    } else if ( !strcmp(AutoCorrFile, "16148s1") ) {
        p_autocorr = & D16148s1;
    } else if ( !strcmp(AutoCorrFile, "16149s1") ) {
        p_autocorr = & D16149s1;
    } else if ( !strcmp(AutoCorrFile, "16150s1") ) {
        p_autocorr = & D16150s1;
    } else if ( !strcmp(AutoCorrFile, "16155s1") ) {
        p_autocorr = & D16155s1;
    } else if ( !strcmp(AutoCorrFile, "16594s1") ) {
        p_autocorr = & D16594s1;
    } else {
        printf("Error in Autocorrelation file input name\n");
        exit(1);
    }


    //XML input data
    XMLDocument doc; //constructor
    XMLDeclaration *decl;
    XMLComment *comment;
    XMLElement *elem, *cemh;
    char str2 [ 4096 ];

    decl = doc.NewDeclaration( "1.0" );
    doc.LinkEndChild(decl);
    
    comment = doc.NewComment("Automatically generated CEMHYD3D XML input file"); 
    doc.LinkEndChild(comment);
    
    cemh = doc.NewElement("cemhyd");
    doc.LinkEndChild(cemh);
       
    elem = doc.NewElement("resolution");
    elem->SetAttribute("width", 1);
    elem->SetAttribute("height", 5);
    cemh->LinkEndChild(elem);
    
    sprintf(str2, "fineness %f PSD-%s dihydrate %f  hemihydrate %f  anhydrite %f AutoCorrFile %s C3S_mass %f  C2S_mass %f  C3A_mass %f  C4AF_mass %f  totsodium %f  totpotassium %f  rssodium %f  rspotassium %f  wcr %f  satur_sealed %d", fineness, Fineness_switch == 0 ? "Dyckerhoff" : "Breugel", dihydrate, hemihydrate, anhydrite, AutoCorrFile, C3S_mass, C2S_mass, C3A_mass, C4AF_mass, totsodium, totpotassium, rssodium,  rspotassium, wcr, satur_sealed);

    comment = doc.NewComment((const char *) str2);
    cemh->LinkEndChild(comment);
    
    elem = doc.NewElement("Rand_seed_num");
    elem->SetAttribute("key0", rand_seed_num);
    cemh->LinkEndChild(elem);

    elem = doc.NewElement("Microstructure_size");
    elem->SetAttribute("key0", SYSSIZE);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Generate_microstructure");
    sprintf(str2, "key%d", cnt++);
    elem->SetAttribute(str2, 2);
    sprintf(str2, "key%d", cnt++);
    elem->SetAttribute(str2, truncated_radius + 1);
    sprintf(str2, "key%d", cnt++);
    elem->SetAttribute(str2, 0);                          //dispersion
    elem->SetAttribute("dihydrate", dihydrate);
    elem->SetAttribute("hemihydrate", hemihydrate);
    elem->SetAttribute("anhydrite", anhydrite);
    for ( i = truncated_radius; i >= 0; i-- ) {
        sprintf(str2, "key%d", cnt++);
        elem->SetAttribute(str2, ( int ) Particles [ i ]);
        sprintf(str2, "key%d", cnt++);
        elem->SetAttribute(str2, i);
        sprintf(str2, "key%d", cnt++);
        elem->SetAttribute(str2, 1);
    }

    sprintf(str2, "key%d", cnt++);
    elem->SetAttribute(str2, 1);
    elem->SetAttribute("C3S_unit_frac", clinker_unity_frac [ 0 ]);
    elem->SetAttribute("C2S_unit_frac", clinker_unity_frac [ 1 ]);
    elem->SetAttribute("C3A_unit_frac", clinker_unity_frac [ 2 ]);
    elem->SetAttribute("C4AF_unit_frac", clinker_unity_frac [ 3 ]);
    //autocorrelation files
    sprintf(str2, "key%d", cnt++);
    elem->SetAttribute(str2, 60);
    for ( i = 0; i < 60; i++ ) {
        sprintf(str2, "key%d", cnt++);
        elem->SetAttribute(str2, i);
        sprintf(str2, "key%d", cnt++);
        elem->SetAttribute(str2, p_autocorr->sil [ i ]);
    }

    sprintf(str2, "key%d", cnt++);
    elem->SetAttribute(str2, 60);

    for ( i = 0; i < 60; i++ ) {
        sprintf(str2, "key%d", cnt++);
        elem->SetAttribute(str2, i);
        sprintf(str2, "key%d", cnt++);
        elem->SetAttribute(str2, p_autocorr->c3s [ i ]);
    }

    sprintf(str2, "key%d", cnt++);
    elem->SetAttribute(str2, 60);
    for ( i = 0; i < 60; i++ ) {
        sprintf(str2, "key%d", cnt++);
        elem->SetAttribute(str2, i);
        sprintf(str2, "key%d", cnt++);
        elem->SetAttribute(str2, p_autocorr->c3a [ i ]);
    }

    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Total_sodium");
    elem->SetAttribute("key0", totsodium);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Total_potassium");
    elem->SetAttribute("key0", totpotassium);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Readily_soluble_sodium");
    elem->SetAttribute("key0", rssodium);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Readily_soluble_potassium");
    elem->SetAttribute("key0", rspotassium);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Diffusion_steps_per_cycle");
    elem->SetAttribute("key0", 500);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("CH_nucleation_probability");
    elem->SetAttribute("key0", 0.0001);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("CH_scale_factor");
    elem->SetAttribute("key0", 9000.);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Gypsum_nucleation_probability");
    elem->SetAttribute("key0", 0.01);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Gypsum_scale_factor");
    elem->SetAttribute("key0", 9000.);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("C3AH6_nucleation_probability");
    elem->SetAttribute("key0", 0.00002);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("C3AH6_scale_factor");
    elem->SetAttribute("key0", 10000.);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("FH3_nucleation_probability");
    elem->SetAttribute("key0", 0.002);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("FH3_scale_factor");
    elem->SetAttribute("key0", 2500.);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Given_microstructure");
    elem->SetAttribute("key0", 0);
    elem->SetAttribute("note0", "0-not present, 1-present");
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Input_id_file");
    elem->SetAttribute("key0", "out_mic.id");
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Input_img_file");
    elem->SetAttribute("key0", "out_mic.img");
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Output_initial_microstructure");
    elem->SetAttribute("key0", 1);
    elem->SetAttribute("note0", "0-do not output, 1-output");
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Output_initial_microstructure_id_file");
    elem->SetAttribute("key0", "out_mic.id");
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Output_initial_microstructure_img_file");
    elem->SetAttribute("key0", "out_mic.img");
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Saturated_sealed");
    elem->SetAttribute("key0", satur_sealed);
    elem->SetAttribute("note0", "0-saturated, 1-sealed");
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Cycle_freq_perc_pore");
    elem->SetAttribute("key0", 100);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Cycle_freq_perc_sol");
    elem->SetAttribute("key0", 50000);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Induction_time");
    elem->SetAttribute("key0", dormant);
    elem->SetAttribute("note0", "Extra dormant period");
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Beta");
    elem->SetAttribute("key0", time_mapping);
    elem->SetAttribute("note0", "Parameter for time-cycle mapping");
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Ea_cement");
    elem->SetAttribute("key0", Ea_cem); //kJ/mol
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Ea_pozz");
    elem->SetAttribute("key0", 83.14); //kJ/mol
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Ea_slag");
    elem->SetAttribute("key0", 80.0); //kJ/mol
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Water_cement_ratio");
    elem->SetAttribute("key0", wcr); //[-]
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Mass_tot_concrete");
    elem->SetAttribute("key0", mass_tot); //kg
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Mass_cement_concrete");
    elem->SetAttribute("key0", mass_cem); //kg
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Mass_cem_frac");
    elem->SetAttribute("key0", mass_cem / mass_tot); //[-]
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Mass_water_frac");
    elem->SetAttribute("key0", vol_wat*1.0 / mass_tot); //[-]
    cemh->LinkEndChild(elem);
        elem = doc.NewElement("Mass_SCM_frac");
    elem->SetAttribute("key0", mass_SCM / mass_tot); //[-]
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Mass_FA_frac");
    elem->SetAttribute("key0", mass_FA / mass_tot); //[-]
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Mass_CA_frac");
    elem->SetAttribute("key0", mass_CA / mass_tot); //[-]
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Mass_inert_frac");
    elem->SetAttribute("key0", mass_inert / mass_tot); //[-]
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Mass_SCM_FA_CA_inert_frac");
    elem->SetAttribute("key0", ( mass_SCM + mass_FA + mass_CA + mass_inert ) / mass_tot); //mass_agg contains SCM + FA + CA + filler (for heat capacity), SCM not included in CEMHYD3D's UC
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Cp_cem");
    elem->SetAttribute("key0", Cp_cem);// [J/kg/K]
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Cp_SCM_FA_CA_inert");//used only in standalone CEMHYD3D [J/kg/K]
    elem->SetAttribute("key0", Cp_agg);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Cp_SCM");// [J/kg/K]
    elem->SetAttribute("key0",Cp_SCM);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Cp_FA");// [J/kg/K]
    elem->SetAttribute("key0", 800.);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Cp_CA");// [J/kg/K]
    elem->SetAttribute("key0", 800.);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Cp_inert");// [J/kg/K]
    elem->SetAttribute("key0", 800.);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Adiabatic_conditions");
    elem->SetAttribute("key0", strcmp("Off", adiabatic_curing) == 0 ? 0 : 1);
    elem->SetAttribute("note0", "0-no adiabatic curing, 1-adiabatic curing");
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Vol_total");
    elem->SetAttribute("key0", vol_tot);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Vol_cement_clinker_gypsum");
    elem->SetAttribute("key0", vol_cem);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Vol_cement_SCM");
    elem->SetAttribute("key0", vol_SCM);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Vol_water");
    elem->SetAttribute("key0", vol_wat);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Vol_FA");
    elem->SetAttribute("key0", vol_FA);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Vol_CA");
    elem->SetAttribute("key0", vol_CA);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Vol_inert_filler");
    elem->SetAttribute("key0", vol_inert);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Vol_entrained_entrapped_air");
    elem->SetAttribute("key0", vol_entrained_entrapped);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Grain_average_FA");
    elem->SetAttribute("key0", grain_FA);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Grain_average_CA");
    elem->SetAttribute("key0", grain_CA);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("ITZ_thickness");
    elem->SetAttribute("key0", ITZ_thick);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("ITZ_Young_red");
    elem->SetAttribute("key0", ITZ_Young_red);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Young_SCM");
    elem->SetAttribute("key0", E_SCM);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Poisson_SCM");
    elem->SetAttribute("key0", nu_SCM);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Young_FA");
    elem->SetAttribute("key0", E_FA);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Poisson_FA");
    elem->SetAttribute("key0", nu_FA);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Young_CA");
    elem->SetAttribute("key0", E_CA);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Poisson_CA");
    elem->SetAttribute("key0", nu_CA);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Young_inert");
    elem->SetAttribute("key0", E_inert);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Poisson_inert");
    elem->SetAttribute("key0", nu_inert);
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Calculate_elastic_homogenization");
    elem->SetAttribute("key0", 1);
    elem->SetAttribute("note0", "0-do not calculate elastic homogenization, 1-calculate");
    cemh->LinkEndChild(elem);
    //for OOFEM
    elem = doc.NewElement("Concrete_bulk_density");// [kg/m3]
    elem->SetAttribute("key0", mass_tot/vol_tot*1000.);
    elem->SetAttribute("note0", "[kg/m3]");
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Concrete_thermal_conductivity");// [W/m/K]
    elem->SetAttribute("key0", 1.7);
    elem->SetAttribute("note0", "Thermal conductivity in the beginning of hydration [W/m/K]");
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Info_concrete_thermal_capacity_DoH_0");// [J/kg/K]
    capacity_paste = (4180 * vol_wat*1.0 + Cp_cem * mass_cem ) / mass_tot;//cement paste
    capacity_concrete = (Cp_SCM*mass_SCM + Cp_FA*mass_FA + Cp_CA*mass_CA + Cp_inert*mass_inert)/mass_tot;
    elem->SetAttribute("key0", capacity_paste+capacity_concrete);
    elem->SetAttribute("note0", "Informative heat capacity at DoH=0 [J/kg/K]");
    cemh->LinkEndChild(elem);
    elem = doc.NewElement("Info_concrete_thermal_capacity_DoH_1");// [J/kg/K]
    capacity_paste *= (1.-0.26*(1-pow(2.71828, -2.9*1.0)));
    elem->SetAttribute("key0", capacity_paste+capacity_concrete);
    elem->SetAttribute("note0", "Informative heat capacity at DoH=1 [J/kg/K]");
    cemh->LinkEndChild(elem);
    
    if ( doc.SaveFile("CEMHYD_inp_fileXML.src") ) {
        printf("Output file can not be saved (file %s, line %d)\n", __FILE__, __LINE__);
    }
}



BOOST_PYTHON_MODULE(CEM_toolsXML_py)
{
    boost :: python :: class_< CEM_toolsXML_py >("CEM_toolsXML_py")
    .def("GenerateXMLInputCEMHYD", & CEM_toolsXML_py :: GenerateXMLInputCEMHYD)
    //.def("Get", &CEM_tools_py::Get)
    //.def("Getlist", &CEM_tools_py::Getlist)
    //.def("Setlist", &CEM_tools_py::Setlist)
    ;
}

